__FileName__    =  'stockAnalysis.py'
__CreatedDate__ = '2021/03/08 16:20:12'
__Author__      =  'Yizhe Zhang'
__WebSite__     = 'http://ervinzhang.pythonanywhere.com/'

import pandas as pd
import mplfinance as mpf
from datetime import datetime
import numpy as np
import matplotlib.pyplot as plt

def dateProcess(startDate,endDate,data):
    if startDate=="":
        startDate = data.index.min()
    if endDate=="":
        endDate = data.index.max()
    if type(startDate) != pd._libs.tslibs.timestamps.Timestamp:
        startDate = datetime.strptime(startDate,'%Y-%m-%d')
    if type(endDate) != pd._libs.tslibs.timestamps.Timestamp:
        endDate = datetime.strptime(endDate,'%Y-%m-%d')
    return startDate,endDate

def dataConcat(stock1,stock2):
    data = pd.concat([stock1["Close"],stock2["Close"]],axis=1)
    data.columns = ["stock1","stock2"]
    return data

def plotKLine(data,movingAvg=(3,6,9),plotType="candle",startDate="",endDate=""):
    """
        data: pd.DataFrame with Date(index), Open/High/Low/Close/Volume(Columns)
        movingAvg: (period1,period2,period3...) or period1   # 移动平均时间
        plotType: "candle" / "line" / "renko" / "pnf"
        startDate: "YYYY-MM-DD"
        endDate: "YYYY-MM-DD"
    """
    try:
        startDate,endDate = dateProcess(startDate,endDate,data)
        data2 = data[["Open","High","Low","Close","Volume"]][startDate:endDate]
        mpf.plot(data2,type=plotType,mav=movingAvg,volume=True,show_nontrading=True)
    except KeyError:
        print("KeyError: 请检查数据是否包含Date索引+OHLCV五列")
    except ValueError:
        print("ValueError: 请检查开始、结束日期格式是否为YYYY-MM-DD")
    except TypeError:
        print('TypeError: 请检查图表类型是否为"candle" / "line" / "renko" / "pnf"')
        print('TypeError: 请检查移动平均是否为(period1,period2,period3...)或period1')
    except Exception:
        print("Something wrong with stock data")

def regressionAnalysis(stock1,stock2,startDate="",endDate="",regressDeg=1):
    """
        stock1/stock2: pd.DataFrame with Date(index), Open/High/Low/Close/Volume(Columns)
        startDate: "YYYY-MM-DD"
        endDate: "YYYY-MM-DD"
        regressDeg: num  # 线性回归的次方
    """

    # 数据合并
    data = dataConcat(stock1,stock2)
    # 日期处理
    startDate,endDate = dateProcess(startDate,endDate,data)
    # 数据趋势图
    print("(1)数据趋势图")
    data.loc[startDate:endDate].plot(secondary_y="stock2",figsize=(10,6))
    plt.show()
    # 股票之间变化情况
    rst = np.log(data/data.shift(1))
    # kde图
    print("(2)散点图/KDE图")
    pd.plotting.scatter_matrix(rst,alpha=0.2,diagonal="kde",figsize=(10,6))
    plt.show()
    # 回归方程(一次函数)
    # 去除NaN
    rst.dropna(inplace=True)
    # 求回归系数
    reg = np.polyfit(rst["stock1"],rst["stock2"],deg=regressDeg)
    # 散点图
    ax = rst.plot(kind="scatter",x='stock1',y='stock2',figsize=(10,6))
    # 直线一次函数
    print("(3)线性回归图")
    ax.plot(rst["stock1"],np.polyval(reg,rst["stock1"]),"r")
    plt.show()

def correlationAnalysis(stock1,stock2,startDate="",endDate="",movingAvg=5):
    """
        stock1/stock2: pd.DataFrame with Date(index), Open/High/Low/Close/Volume(Columns)
        startDate: "YYYY-MM-DD"
        endDate: "YYYY-MM-DD"
        movingAvg: num
    """

    if movingAvg>len(stock1)-1:
        return "Moving Average Exceed!"
    # 数据合并
    data = dataConcat(stock1,stock2)
    # 日期处理
    startDate,endDate = dateProcess(startDate,endDate,data)
    # 数据与前一天相除
    rst = np.log(data/data.shift(1))
    # 去除NaN
    rst.dropna(inplace=True)
    # 相关系数
    print("(1)相关系数")
    print(rst.corr())
    # 随年份变化，计算相关系数的变化情况
    print("(2)相关系数随年份变化的情况")
    rst["stock1"].rolling(movingAvg).corr(rst["stock2"]).plot(figsize=(10,6))
    plt.show()


__FileName__    =  'stockKnowledge.py'
__CreatedDate__ = '2021/03/08 17:24:34'
__Author__      =  'Yizhe Zhang'
__WebSite__     = 'http://ervinzhang.pythonanywhere.com/'

def getStockKnowledge():
    return """
    股票:凭证
    作用: 证明+分红
    上市/IPO: 企业通过证券交易所公开向社会发股票募集资金
    按业绩分类: 蓝筹股/绩优股/ST股
    按上市地区分类: A股(人民币,T+1,涨跌10%)、B股(外币,交割T+1,收益T+3)、H股(香港,T+0,无涨跌限制)、N股(纽约)、S股(新加坡)
    股票市场构成: 上市公司、投资者、证监会、证券业协会、交易所、证券中介机构
    交易所: 上证所(主板-沪指)、深证所(主板-深成指,中小板,创业板)
    股价影响因素: 公司自身因素、行业因素、市场因素、心理因素、经济因素、政治因素
    A股股票买卖:
        在券商开户委托购买
        交易日(1-5)
        交易时间:
            9:15-9:25开盘集合竞价
            9:30-11:30前市，连续竞价时间
            13:00-15:00后市，连续竞价时间
            14:57-15:00深交所收盘集合竞价时间
        T+1交易
        涨跌限制10%
    """

def getFinanceAnalysisKnowledge():
    return """
    基本面分析
        宏观经济分析: 国家财政政策、货币政策
        行业分析
        公司分析: 财务数据、业绩报告等
    技术面分析
        K线
            红柱(涨):阳线(柱顶部收盘价，柱下部开盘价，线顶部最高价，线底部最低价)
            绿柱(跌):阴线(柱顶部开盘价，柱下部收盘价，线顶部最高价，线底部最低价)
        MA(均线)
        KDJ(随机指标)
        MACD(指数平滑移动平均线)
    """

def getQuantitativeInvestmentKnowledge():
    return """
        概念
            量化投资: 数学模型实现投资策略
            优势: 客观、多角度、跟踪市场发现新模型、决定策略后可回测验证
            量化策略: 固定逻辑分析/判断/决策，自动化交易
            策略周期: 产生想法/学习知识->python实现->回测/模拟交易->实盘交易->优化策略/放弃策略
            输入(行情数据+财务数据+自定义数据+投资经验)
            策略(选股->择时->仓位管理->止盈止损)
            输出(买入信号+卖出信号+交易费用+收益)
        工具
            平台: 聚宽、优矿、米筐、quantopian
            框架: RQAlpha、QUANTAXIS
        ipython
            tab补全
            ?内省
            ！执行系统命令
            %run执行，%paste执行剪贴板，%timeit运行时间，%pdb自动调试
    """


#!/usr/bin/env python
from setuptools import setup, find_packages
import sys

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="ezfintech",
    version="0.0.31",
    author="Yizhe Zhang",
    author_email="ervinzhang319@gmail.com",
    description="get stock data from different APIs and Analyze Stock Data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/YizheZhang-Ervin/EZFintech",
    packages=find_packages(),
    install_requires=["numpy","pandas","matplotlib","requests","mplfinance"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



__FileName__    =  'getData.py'
__CreatedDate__ = '2021/03/08 11:21:21'
__Author__      =  'Yizhe Zhang'
__WebSite__     = 'http://ervinzhang.pythonanywhere.com/'

import pandas as pd
import requests

def getStockByDay(stockCode,day="today",source="126"):
    """
        web: 126
        stockCode: "XXXXXXX"  # 7位代码
        day: "today" / "4d"   # 日数据 / 4日数据
    """
    try:
        if len(stockCode)<7:
            stockCode = "0"+stockCode

        url = f"http://img1.money.126.net/data/hs/time/{day}/{stockCode}.json"
        r = requests.get(url)
        rst = eval(r.text)
        rst2 = pd.DataFrame(rst["data"])
        # 小时分钟时间、价格、均价、成交量
        rst2.columns = ["time","price","AVG price","Volume"]
        rst2.set_index("time",inplace=True)
        return rst2
    except Exception:
        print("Stock Parameters Error or Stock Server Error")

def getStockByYear(stockCode,year="2020",adjust="klinederc",source="126"):
    """
        web: 126
        stockCode: "XXXXXXX"           # 7位代码
        year: "YYYY"                   # 年份
        adjust: "kline" / "klinederc"  # 不复权 / 复权
    """
    try:
        # 年数据
        if len(stockCode)<7:
            stockCode = "0"+stockCode

        url = f"http://img1.money.126.net/data/hs/{adjust}/day/history/{year}/{stockCode}.json"
        r = requests.get(url)
        rst = eval(r.text)
        rst2 = pd.DataFrame(rst["data"])
        rst2.columns = ["Date","Open","High","Low","Close","Volume","Rate"]
        rst2.set_index("Date",inplace=True)
        rst2.index =pd.to_datetime(rst2.index)
        return rst2
    except Exception:
        print("Stock Parameters Error or Stock Server Error")

def getStockByPeriod(stockCode,period="week",adjust="klinederc",source="126"):
    """
        web: 126 / StockStar
        stockCode: "XXXXXX"               # 7位代码
        period: "day" / "week" / "month"  # 周期
        adjust: "kline" / "klinederc"     # 不复权 / 复权
    """
    try:
        if len(stockCode)<7:
            stockCode = "0"+stockCode

        # 周期数据
        if source=="126":
            url = f"http://img1.money.126.net/data/hs/{adjust}/{period}/times/{stockCode}.json"
            r = requests.get(url)
            rst = eval(r.text)
            rst2 = pd.DataFrame.from_dict(rst)
            rst2 = rst2[["times","closes"]]
            rst2.set_index("times",inplace=True)
            rst2.index.name = "Date"
            rst2.index =pd.to_datetime(rst2.index)
            return rst2

        elif source=="stockstar":
            shanghai = ["60","900","730","700"]
            shenzhen = ["00","200","080","002","300"]
            if stockCode.startswith(shanghai[0]) or stockCode.startswith(shanghai[1]) or stockCode.startswith(shanghai[2]) or stockCode.startswith(shanghai[3]):
                market = 1  # 市场1表示沪
            elif stockCode.startswith(shenzhen[0]) or stockCode.startswith(shenzhen[1]) or stockCode.startswith(shenzhen[2]) or stockCode.startswith(shenzhen[3]):
                market = 2  # 市场2表示深
            
            if period=="day":periodNum=6
            if period=="week":periodNum=7
            if period=="month":periodNum=8

            url = f"http://cq.ssajax.cn/interact/getTradedata.ashx?pic=qlpic_{stockCode}_{market}_{periodNum}"

            r = requests.get(url)
            rst = eval(r.text[31:-1].replace("true","True"))
            rst2 = pd.DataFrame(rst["datas"])
            rst2.columns = ["Date","1","2","3","4","5","6","7","8","9"]
            rst2.set_index("Date",inplace=True)
            rst2.index =pd.to_datetime(rst2.index)
            return rst2
        else:
            return "No this source"
    except Exception:
        print("Stock Parameters Error or Stock Server Error")

def getStockByTwoDate(stockCode,startDate="20210101",endDate="20210202",source="sohu"):
    """
        web: sohu
        stockCode: "XXXXXX"      # 6位代码
        startDate:"YYYYMMDD"     # 开始日期YYYYMMDD
        endDate:"YYYYMMDD"       # 结束日期YYYYMMDD
    """
    try:
        if len(stockCode)>6:
            stockCode = stockCode[1:]

        # 时间段数据
        url = f"http://q.stock.sohu.com/hisHq?code=cn_{stockCode}&start={startDate}&end={endDate}&stat=1&order=D&period=d&callback=historySearchHandler&rt=jsonp"
        r = requests.get(url)
        rst = eval(r.text[22:-3])
        rst2 = pd.DataFrame(rst["hq"])
        rst2.columns = ["time","1","2","3","4","5","6","7","8","9"]
        rst2.set_index("time",inplace=True)
        rst2.index =pd.to_datetime(rst2.index)
        return rst2
    except Exception:
        print("Stock Parameters Error or Stock Server Error")